==> 0.8.0.tar.gz: lib/src/model/value_transform.dart <==
part of fields.model;

final _valueTransforms = new Set<ValueTransform>.from(const [
  const ValueTransform<double, num>(_double2num),
  const ValueTransform<int, num>(_int2num),
  const ValueTransform<num, double>(_num2double),
  const ValueTransform<num, int>(_num2int),
  const ValueTransform<String, int>(_string2int),
  const ValueTransform<String, double>(_string2double)
]);

final Set<ValueTransform> valueTransforms = new Set.from(_valueTransforms);

num _double2num(double value) => value;
num _int2num(int value) => value;
double _num2double(num value) => value.toDouble();
int _num2int(num value) => value.toInt();

int _string2int(String value) {
  try {
    return int.parse(value);
  } on FormatException catch(e) {
    throw new ValueTransformError(cause: e);
  }
}

double _string2double(String value) {
  try {
    return double.parse(value);      
  } on FormatException catch(e) {
    throw new ValueTransformError(cause: e);
  }
}

/// An error encountered during a transform.
class ValueTransformError extends Error {
  
  /// The caught object that caused the transform to err; may be `null` if none.
  final cause;
  
  /// A description of this transform error.
  final String message;
  
  /// Constructs a new transform error with the optional [cause] and [message].
  ValueTransformError({this.cause, this.message: ''});
  
  String toString() => '$ValueTransformError: $message |> $cause';
}

/// A transform from type [S] to type [T].
class ValueTransform<S, T> {
  
  /// Check if a transform from the [sourceType] to the [targetType] can be 
  /// performed.  
  /// 
  /// Returns `true` if such a transform is possible, else `false`.
  static bool canTransform(Type sourceType, Type targetType) {
    if (sourceType == targetType) return true;
    return valueTransforms.any((t) => t.sourceType == sourceType && 
        t.targetType == targetType);
  }
  
  /// Transform the given [value] from its [sourceType] to a [targetType] and
  /// returns the transformed value.
  static transform(value, Type sourceType, Type targetType) {
    if (sourceType == targetType) return value;
    // TODO(rms): check `isAssignableTo` rather than direct equality?
    return valueTransforms.firstWhere((t) => t.sourceType == sourceType && 
        t.targetType == targetType)(value);
  }
  
  final Function _transform;
  final Function _inverse;
  
  /// The source type of this transform.
  Type get sourceType => S;
  
  /// The target type of this transform.
  Type get targetType => T;
      
  /// Whether or not this transform has an [inverse] function.
  bool get hasInverse => _inverse != null;
  
  /// Creates a new field transform for the given [transform] function and 
  /// optional [inverse] function.
  /// 
  /// If the [inverse] function is not `null` then this transform is a two-way
  /// transform; else it is a one-way transform.
  const ValueTransform(T transform(S), [S inverse(T)]) 
  : _transform = transform
  , _inverse = inverse;
  
  /// Call this transform on [value] and returns the transformed value.
  T call(S value) => _transform(value);
  
  /// Call this transform's inverse on [value] and returns the transformed 
  /// value.
  /// 
  /// If [hasInverse] is false this method will throw an [UnsupportedError].
  S inverse(T value) {
    if (!hasInverse) {
      throw new UnsupportedError('$runtimeType does not have an inverse.');
    }
    return _inverse(value);
  }
}

==> 0.8.4.tar.gz: lib/src/model/value_transform.dart <==
part of fields.model;

final _valueTransforms = new Set<ValueTransform>.from(const [
  const ValueTransform<double, num>(_double2num),
  const ValueTransform<int, num>(_int2num),
  const ValueTransform<num, double>(_num2double),
  const ValueTransform<num, int>(_num2int),
  const ValueTransform<String, int>(int.parse),
  const ValueTransform<String, double>(double.parse)
]);

final Set<ValueTransform> valueTransforms = new Set.from(_valueTransforms);

num _double2num(double value) => value;
num _int2num(int value) => value;
double _num2double(num value) => value.toDouble();
int _num2int(num value) => value.toInt();

/// A transform from type [S] to type [T].
class ValueTransform<S, T> {
  
  /// Check if a transform from the [sourceType] to the [targetType] can be 
  /// performed.  
  /// 
  /// Returns `true` if such a transform is possible, else `false`.
  static bool canTransform(Type sourceType, Type targetType) {
    if (sourceType == targetType) return true;
    return valueTransforms.any((t) => t.sourceType == sourceType && 
        t.targetType == targetType);
  }
  
  /// Transform the given [value] from its [sourceType] to a [targetType] and
  /// returns the transformed value.
  static transform(value, Type sourceType, Type targetType) {
    if (sourceType == targetType) return value;
    return valueTransforms.firstWhere((t) => t.sourceType == sourceType && 
        t.targetType == targetType)(value);
  }
  
  final Function _transform;
  final Function _inverse;
  
  /// The source type of this transform.
  Type get sourceType => S;
  
  /// The target type of this transform.
  Type get targetType => T;
      
  /// Whether or not this transform has an [inverse] function.
  bool get hasInverse => _inverse != null;
  
  /// Creates a new field transform for the given [transform] function and 
  /// optional [inverse] function.
  /// 
  /// If the [inverse] function is not `null` then this transform is a two-way
  /// transform; else it is a one-way transform.
  const ValueTransform(T transform(S), [S inverse(T)]) 
  : _transform = transform
  , _inverse = inverse;
  
  /// Call this transform on [value] and returns the transformed value.
  T call(S value) => _transform(value);
  
  /// Call this transform's inverse on [value] and returns the transformed 
  /// value.
  /// 
  /// If [hasInverse] is false this method will throw an [UnsupportedError].
  S inverse(T value) {
    assert(hasInverse);
    return _inverse(value);
  }
}
